### 
#
# Date:    30.09.2020
# Titel:   Geometric Morphometric Analysis of Ceratitidae - Developmental morphospaces with 2D plots, subsequent trajectory analysis 
#          and shape visualisations
# Comment: Before this file the script 1_Script_GMM_Ceratitid_INPUT must be executed, which contains the data input. This can be done manually 
#          or through the source-command.
#
###

#### 0. Packages ####
#install.packages("geomorph")   #GMM analysis
#install.packages("RRPP")       #GMM analysis
#install.packages("Morpho")     #GMM analysis
#install.packages("ggplot2")    #tidyverse: Plot
#install.packages("ggrepel")    #tidyverse: Text aestethics (geom_text_repel & geom_label_repel)
#install.packages("plotly")     #tidyverse: transparency in plots(alpha)
#install.packages("scales")     #tidyverse: Plot aesthetics
#install.packages("dplyr")      #tidyverse: Data manipulation
#install.packages("ggforce")    #tidyverse: 
#install.packages("patchwork") #tidyverse: Combine ggplpts
#install.packages("devtools")
#devtools::install_github("fruciano/GeometricMorphometricsMix", dependencies = TRUE) # Reverse PCA (Visualisation)
#install.packages("rgl")        #interactive 3D Plot
#install.packages("geometry")   #Add a hull in 3D plots


#### 1. Data  #####
setwd("C:/Eva/Texte_Paper/20.07_GMM/Scripts_&_Data")
source("1_Script_GMM_Ceratitid_INPUT.R")  # The data file can be read manuallyor directly in this script 

### 2. DEVELOPMENTAL Morphospace ######## 

# 2.1. PCA #####
#library(Morpho)
Shape.ma <- Morpho::vecx(Shape.df) # prcomp only works with matrices
pca<-prcomp(Shape.ma, center = TRUE, retx = TRUE, scale. = FALSE) 

## Calculate variances
pca.var <- pca$sdev^2; pca.var #square of standard devation (variance)
pca.var.per <- round(pca.var/sum(pca.var)*100, 1); pca.var.per
pca.scores <- c(pca.var.per[1], pca.var.per[2], pca.var.per[3]); pca.scores

### 2.2. GGPLOT #####

## Create data frame for ggplot
{df$PCA <- as.data.frame(pca$x[,1:3]) 
row.names(df$PCA ) <- list$ind #Naming rows of df$PCA
df$PCA$species <- list$species
df$PCA$genus <- list$genus
df$PCA$species.occ<-list$species.occ
df$PCA$whorl <- list$whorl
}


## Plot
library(ggplot2) # plot function
library(ggrepel) # For geom_text_repel & geom_label_repel
library(scales)  # for transparency in plots(alpha)
 
{
windows(width=6.5, height=5.5)
p <- ggplot2::ggplot(data=df$PCA, aes(x = PC1, y = PC2, colour=species, fill = species))+theme_light()

 p <- p+ geom_vline(xintercept = 0, colour="darkgrey")+
         geom_hline(yintercept = 0, colour="darkgrey")+
         geom_point(aes(text=paste(list$ind.pure, list$species, sep = " - ")), size=list$whorl/3.5)+ # Text for plotly
         scale_fill_manual(values=Palette)+scale_colour_manual(values=Palette) #Color palette defined above

 # Theme  
 p <- p + theme(legend.title=element_blank(),
                plot.title = element_text(hjust = 0.5, color="black", size=12, face="bold"),
                plot.subtitle = element_text(hjust = 0.5, size=10, face="bold"),
                axis.title.x = element_text(color="black", size=12, face="bold"),
                axis.title.y = element_text(color="black", size=12, face="bold"), 
                plot.caption = element_text(hjust = 0))

# Define more optics 
 p <- p + guides(color = guide_legend(override.aes = list(size=5))) #Size of legend symbols
 p <- p + coord_fixed(ratio = 1, xlim = c(min(df$PCA[,1]),max(df$PCA[,1])), c(min(df$PCA[,2]),max(df$PCA[,2])), expand = TRUE, clip = "on")
 p <- p + scale_x_continuous(breaks = round(seq(round(min(df$PCA[,1]), 1), round(max(df$PCA[,1]), 1), by = 0.05),2))+
          scale_y_continuous(breaks = round(seq(round(min(df$PCA[,1]), 1), round(max(df$PCA[,1]), 1), by = 0.05),2))
  
#Specialized labs    
 p <- p+#labs(caption = paste ("PC1+PC2: ", pca.scores[1]+pca.scores[2], " %; ", Sys.Date(), sep=""))+
        xlab(paste("PC1", " - ", pca.var.per[1], "%", sep="")) +
        ylab(paste("PC2", " - ", pca.var.per[2], "%", sep=""))
 

#Facets
 p <- p + facet_wrap(~species.occ, ncol = 3)+theme(legend.position = 'none') 
  
# p <- p + facet_grid(species.occ~df$PCA$whorl)+theme(legend.position = 'none', strip.text.y.right = element_text(angle=0))+
#          geom_vline(aes(xintercept = 0), size=0.5,alpha=0.4)+geom_hline(aes(yintercept = 0), size=0.5, alpha=0.4)
  
Plots$PCA <- p; Plots$PCA
#ggsave(paste("P_Dev-Morphospace_", Sys.Date(),".jpeg", sep=""), dpi=400)
}
#dev.off()

## Extracting the no. of an individual in the tps.file
#(which(list$ind.pure == "C10137")/11)[11]


### 2.3. Interactive plot #####
library(plotly)
Plots$PCAly <- plotly::ggplotly(Plots$PCA); Plots$PCAly


### 2.5. Barplot ####
#last 4 dimensions are lost because of fGPA
{windows(width=6.5, height=5.5)
barplot((pca$sdev^2)/sum(pca$sdev^2),
        main=paste(filterName, sep=""),
        sub= paste (DataSet, "; ", Sys.Date(), sep=""),
        xlab="Principal Component",
        ylab="Percent Variation") +
mtext(paste("Barplot -  ",DataSet, sep=""))
#savePlot(paste("P_Barplot", filterName, LMType, Sys.Date(),".jpeg", sep="_"), type="jpeg")
}



#### 3. TPS splines ####

### 3.1. Data preparation #####

## Calculate mean shape (needed for PCAreversion)
MeanShape = matrix(colMeans(Shape.ma), nrow=16, ncol=2) # use the mean of the columns of Shape.ma
MeanShape_5.5 = matrix(colMeans(Shape.ma[list$whorl==5.5,]), nrow=16, ncol=2) # use the mean of the columns of Whorl 5.5
MeanShape_0.5 = matrix(colMeans(Shape.ma[list$whorl==0.5,]), nrow=16, ncol=2) 
MeanShape_3.0 = matrix(colMeans(Shape.ma[list$whorl==3.0,]), nrow=16, ncol=2) 


## Get extreme values
Extremes_PC1_PC2=as.matrix(data.frame(PC1=range(pca$x[,1][(list$whorl=="0.5") | (list$whorl=="5.5")]),  #most extreme values on PC1 of whorl 0.5&5.5
                                      PC2=range(pca$x[,2][(list$whorl=="0.5") | (list$whorl=="5.5")]))) #most extreme values on PC2 of whorl 0.5&5.5
rownames(Extremes_PC1_PC2)=c("minPCs", "maxPCs"); Extremes_PC1_PC2


## Buildmatrix with Min/Max-values
AllPCMinMax<-matrix(c(
    c(Extremes_PC1_PC2[2,1],Extremes_PC1_PC2[2,2]),  # MaxPC1_MaxPC2
    c(Extremes_PC1_PC2[1,1],Extremes_PC1_PC2[2,2]),  # MinPC1_MaxPC2 
    c(Extremes_PC1_PC2[1,1],Extremes_PC1_PC2[1,2]),  # MinPC1_MinPC2
    c(Extremes_PC1_PC2[2,1],Extremes_PC1_PC2[1,2])), # MaxPC1_MinPC2
    ncol=2,byrow = TRUE)
    
rownames(AllPCMinMax)=c("MaxPC1_MaxPC2", "MinPC1_MaxPC2", "MinPC1_MinPC2", "MaxPC1_MinPC2")
colnames(AllPCMinMax)=c("PC1", "PC2")
AllPCMinMax

### 3.2. PCA reversion ####
# Going back from scores to shapes
# Instead of plotting the shape variation described by individual axes
# the variation at the maximum observed scores on PC1 and PC2 
# against the minimum observed scores on both PC1 and PC2 is plotted

## Calculation of Shapes
Pred.Shape=GeometricMorphometricsMix::reversePCA(
  Scores=AllPCMinMax,
  Eigenvectors=pca$rotation[,1:2], # principal components (eigenvectors)
  Mean=MeanShape)

# Conversion to right format
Predicted_Shapes_3D=Morpho::vecx(Pred.Shape, lmdim=2, revert=TRUE, byrow=FALSE) 


### 3.3. Thin-Plate Splines  ####
## 3.3.1. Extreme values of Morphospace ####
{
  windows(width=24, height=24);par(mfrow=c(2,2), mar=c(0,0,1,0)) #unten, links, oben, rechts
  magnification=0.7
  Plots$TPS_MinPC1_MaxPC2 <- geomorph::plotRefToTarget(MeanShape, Predicted_Shapes_3D[,,"MinPC1_MaxPC2"], mag=magnification,
                                                       links = wireframe.ma, label = FALSE); title("MinPC1_MaxPC2", cex.main=1.5)
  Plots$TPS_MaxPC1_MaxPC2 <- geomorph::plotRefToTarget(MeanShape, Predicted_Shapes_3D[,,"MaxPC1_MaxPC2"], mag=magnification, 
                                                       links = wireframe.ma, label = FALSE); title("MaxPC1_MaxPC2", cex.main=1.5)
  Plots$TPS_MinPC1_MinPC2 <- geomorph::plotRefToTarget(MeanShape, Predicted_Shapes_3D[,,"MinPC1_MinPC2"], mag=magnification,
                                                       links = wireframe.ma,label = FALSE); title("MinPC1_MinPC2", cex.main=1.5)
  Plots$TPS_MaxPC1_MinPC2 <- geomorph::plotRefToTarget(MeanShape, Predicted_Shapes_3D[,,"MaxPC1_MinPC2"], mag=magnification,
                                                       links = wireframe.ma,label = FALSE); title("MaxPC1_MinPC2", cex.main=1.5)
  #savePlot(paste("P_TPS-MaxMin",".tif", sep=""), type="tiff")
}

### Extreme values of Morphospace 2x2 (illustration purposes)
{windows(width=1.5, height=5.5);par(mfrow=c(2,1), mar=c(0,0,1,0)) #margin: below, left, above, right
  magnification=0.7
  Plots$TPS_MinPC1_MaxPC2 <- geomorph::plotRefToTarget(MeanShape, Predicted_Shapes_3D[,,"MinPC1_MaxPC2"], mag=magnification,
                                                       links = wireframe.ma, label = FALSE)  
  Plots$TPS_MinPC1_MaxPC2 <- geomorph::plotRefToTarget(MeanShape, Predicted_Shapes_3D[,,"MinPC1_MinPC2"], mag=magnification,
                                                       links = wireframe.ma, label = FALSE)
  #savePlot(paste("P_TPS-Min",".tif", sep=""), type="tiff")  
}   

{windows(width=1.5, height=5.5);par(mfrow=c(2,1), mar=c(0,0,1,0)) #margin: below, left, above, right
  magnification=0.7
  Plots$TPS_MaxPC1_MaxPC2 <- geomorph::plotRefToTarget(MeanShape, Predicted_Shapes_3D[,,"MaxPC1_MaxPC2"], mag=magnification, 
                                                       links = wireframe.ma, label = FALSE)   
  
  Plots$TPS_MaxPC1_MinPC2 <- geomorph::plotRefToTarget(MeanShape, Predicted_Shapes_3D[,,"MaxPC1_MinPC2"], mag=magnification,
                                                       links = wireframe.ma,label = FALSE)
#savePlot(paste("P_TPS-Max",".tif", sep=""), type="tiff")
}   


## 3.3.2. Mean shapes of 0.5 / 3.0 / 5.5 #####
{
  windows(width=5, height=2);par(mfrow=c(1,3), mar=c(0,0,1,0)) #unten, links, oben, rechts
  magnification=0.75 ; line=-0.25
  method="points" # or method=tps
  
  geomorph::plotRefToTarget(MeanShape_0.5, MeanShape_0.5, mag=magnification, method = method,
                            links = wireframe.ma, label = FALSE) + mtext(paste("A) whorl 0.5", sep=""), line=line)
  geomorph::plotRefToTarget(MeanShape_3.0, MeanShape_3.0, mag=magnification, method = method,
                            links = wireframe.ma, label = FALSE) + mtext(paste("B) whorl 3.0", sep=""), line=line)
  geomorph::plotRefToTarget(MeanShape_5.5, MeanShape_5.5, mag=magnification, method = method,
                            links = wireframe.ma, label = FALSE) + mtext(paste("C) whorl 5.5", sep=""), line=line)
  #savePlot(paste("P_TPS-Mean_0.5_3.0_5.5",".tif", sep=""), type="tiff")
  
}

### 3.3.3. Mean Adult Shapes #### 
#Calculate mean shapes per species
MeanShape_species <- list( 
# Assign to element     <-        get shape    [get position of values with     (species=x) and whorl 5.5    ]
   BiCorde = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="BiCorde") & (list$whorl=="5.5")])), nrow=16, ncol=2),
   BrVogde = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="BrVogde") & (list$whorl=="5.5")])), nrow=16, ncol=2),
   DiLawso = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="DiLawso") & (list$whorl=="5.5")])), nrow=16, ncol=2),
   MaSpini = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="MaSpini") & (list$whorl=="5.5")])), nrow=16, ncol=2),
   FrNevad = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="FrNevad") & (list$whorl=="5.5")])), nrow=16, ncol=2),
   FrOccid = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="FrOccid") & (list$whorl=="5.5")])), nrow=16, ncol=2),
   GyBlake = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="GyBlake") & (list$whorl=="5.5")])), nrow=16, ncol=2),
   GyMimet = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="GyMimet") & (list$whorl=="5.5")])), nrow=16, ncol=2),
   GyRotel = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="GyRotel") & (list$whorl=="5.5")])), nrow=16, ncol=2),
   GyWeits = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="GyWeits") & (list$whorl=="5.5")])), nrow=16, ncol=2),
   PaDunni = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="PaDunni") & (list$whorl=="5.5")])), nrow=16, ncol=2),
   PaMeeki = matrix(colMeans(Morpho::vecx(Shape.df[,,(list$species=="PaMeeki") & (list$whorl=="5.5")])), nrow=16, ncol=2))

#Plot function
TPStoMean5.5 <- function(M2, mag=magf, method) {
  geomorph::plotRefToTarget(
    M1=MeanShape_5.5,
    M2=M2,
    mag=mag, method = method,
    links = wireframe.ma,label = FALSE)  ## label = TRUE (LM Nummerierung)
}

# Plot
{
  windows(height=3, width=9);par(mfrow=c(2,6), mar=c(0,0,1,0))
  magf=1.5; line=-0.7
  method="points" #TPS oder points

#Type A  
  TPStoMean5.5(M2=MeanShape_species$FrNevad, mag = magf, method = method) +mtext(paste("FrNevad", sep=""), line=line)
  TPStoMean5.5(M2=MeanShape_species$FrOccid, mag = magf, method = method) +mtext(paste("FrOccid", sep=""), line=line)
  TPStoMean5.5(M2=MeanShape_species$MaSpini, mag = magf, method = method) +mtext(paste("MaSpini", sep=""), line=line)
  TPStoMean5.5(M2=MeanShape_species$BrVogde, mag = magf, method = method) +mtext(paste("BrVogde", sep=""), line=line)
  TPStoMean5.5(M2=MeanShape_species$PaMeeki, mag = magf, method = method) +mtext(paste("PaMeeki", sep=""), line=line)

#Type 
  TPStoMean5.5(M2=MeanShape_species$BiCorde, mag = magf, method = method) +mtext(paste("BiCorde", sep=""), line=line)
  TPStoMean5.5(M2=MeanShape_species$DiLawso, mag = magf, method = method) +mtext(paste("DiLawso", sep=""), line=line)
  TPStoMean5.5(M2=MeanShape_species$GyBlake, mag = magf, method = method) +mtext(paste("GyBlake", sep=""), line=line)
  TPStoMean5.5(M2=MeanShape_species$GyMimet, mag = magf, method = method) +mtext(paste("GyMimet", sep=""), line=line)
  TPStoMean5.5(M2=MeanShape_species$GyRotel, mag = magf, method = method) +mtext(paste("GyRotel", sep=""), line=line)
  TPStoMean5.5(M2=MeanShape_species$GyWeits, mag = magf, method = method) +mtext(paste("GyWeits", sep=""), line=line)
  TPStoMean5.5(M2=MeanShape_species$PaDunni, mag = magf, method = method) +mtext(paste("PaDunni", sep=""), line=line)
  
}

#### 4. Trajectory analysis ####

## 4.1. Data frame ####
g.df.TA=geomorph::geomorph.data.frame(
  Shape.ma=Shape.ma,
  whorl=factor(list$whorl),    #Whorls as a factor!
  species=list$species)


## 4.2. Model  ####
Mod.TA <- RRPP::lm.rrpp(Shape.ma ~ species * whorl, data = g.df.TA, iter = 999,
                        print.progress = FALSE); summary(Mod.TA)

## 4.3. trajectory.analysis  ####
Traj.Anal <- RRPP::trajectory.analysis(Mod.TA,
                                       groups = g.df.TA$species, # Define groups
                                       traj.pts = g.df.TA$whorl, # Define clusters
                                       print.progress = TRUE)
## 4.4. Output  ####
## Get Output
TA_SD_summary<-summary(Traj.Anal, attribute = "SD"); TA_SD_summary # Shape differences between trajectories 
TA_MD_summary<-summary(Traj.Anal, attribute = "MD"); TA_MD_summary # Magnitude difference (absolute difference between path distances)
TA_PD_summary<-Traj.Anal$PD; TA_PD_summary$obs # Path distance
TA_TC_summary<-summary(Traj.Anal, attribute = "TC", angle.type = "deg"); TA_TC_summary #Trajectory correlations from every permutation

## Save Output
write.table(TA_SD_summary$summary.table,"Traj.Anal$SD.txt", sep="\t") #Shape differences
write.table(TA_TC_summary$summary.table,"Traj.Anal$TC.txt", sep="\t") #Trajectory correlation
write.table(TA_MD_summary$summary.table,"Traj.Anal$MD.txt", sep="\t") #Magnitude differences
write.table(TA_PD_summary$obs,"Traj.Anal$PD.txt", sep="\t") #Path distances (Absolute magnitude differences)

## 4.5. Plot  ####
{
  windows(width=5.5, height=4.5)
  par(mar=c(5.0, 4.0, 2.0, 6.0), xpd=TRUE) #margin: below, left, above, right
  
  Plots$TA<-plot(Traj.Anal, pch=2, cex=1, bg=as.numeric(g.df.TA$species), col=as.numeric(g.df.TA$species))
  
  RRPP::add.trajectories(Plots$TA, traj.pch =21, 
                   traj.col = as.numeric(unique(as.factor(g.df.TA$species))),
                   traj.lwd = 2,
                   traj.cex = 1)
  
  legend("topright", inset=c(-0.3,-0.11), legend=levels(as.factor(g.df.TA$species)),
         col = 1:list$No.species, pt.bg=1:list$No.species, cex = 0.8, pch = 21)
  }

