### 
#
# Date:    30.09.2020
# Titel:   Geometric Morphometric Analysis of Ceratitidae - Ontogenetic trajectory spaces (2D and 3D plots)
# Comment: Before this file the script 1_Script_GMM_Ceratitid_INPUT must be executed, which contains the data input. This can be done manually 
#          or through the source-command.
###

#### 0. Packages ####
#install.packages("geomorph")   #GMM analysis
#install.packages("RRPP")       #GMM analysis
#install.packages("Morpho")     #GMM analysis
#install.packages("ggplot2")    #tidyverse: Plot
#install.packages("ggrepel")    #tidyverse: Text aestethics (geom_text_repel & geom_label_repel)
#install.packages("plotly")     #tidyverse: transparency in plots(alpha)
#install.packages("scales")     #tidyverse: Plot aesthetics
#install.packages("dplyr")      #tidyverse: Data manipulation
#install.packages("ggforce")    #tidyverse: 
# install.packages("patchwork") #tidyverse: Combine ggplpts
#install.packages("devtools")
#devtools::install_github("fruciano/GeometricMorphometricsMix", dependencies = TRUE) # Reverse PCA (Visualisation)
#install.packages("rgl")        #interactive 3D Plot
#install.packages("geometry")   #Add a hull in 3D plots


#### 1. Data  #####
setwd("C:/Eva/Texte_Paper/20.07_GMM/Scripts_&_Data")
source("1_Script_GMM_Ceratitid_INPUT.R") # The data file can be read manuallyor directly in this script 


#### 2. ONTOGENETIC MORPHOSPACE ####

## 2.1. Data input ####
v<-as.vector(Shape.df); v2<-NULL

for (j in 1:n){
  for (i in 1:(No.LM)){
    v1<- c(v[(j-1)*2*No.LM+i], v[j*2*No.LM-No.LM+i])
    v2<- c(v2, v1)}};rm(i)


## 2.1.2. New Array with joint configurations ####
Shape.onto<-array(NA, dim=c(No.LM_tot,2,n.ind))  #n: number of configurations) 
for (i in 1:n.ind){
  Shape.onto[,,i]<-matrix(v2[(((i-1)*(No.LM_tot*2))+1):(i*((No.LM_tot*2)))], #fill from i*1.number to i*last number   l   
                           No.LM_tot,
                           2,
                           byrow=T) }#; rm(v); rm(v1);rm(v2)

## Check
head(Shape.onto[1:16,,1]==Shape.df[,,1])  
head(Shape.onto[161:176,,n.ind]==Shape.df[,,n])
head(Shape.onto[1:16,,35]==Shape.df[,,375])

### 2.2. Extract metadata ######
# Compile meta data list for JOINT configurations
slist <- list(
  ind.pure=as.character(substring(ID,4,9)), 
  species=as.character(substring(ID,19,25)),
  genus= as.character(substring(ID,19,20)))


## 2.3. PCA (Ontogenetic) ####
Shape.on.ma<-Morpho::vecx(Shape.onto)
pca_onto<-prcomp(Shape.on.ma, center = TRUE, retx = TRUE, scale. = FALSE) 


### 2.3.1 MANOVA ####
# test whether trajectories differ between species
summary(stats::manova(pca_onto$x[,1:60]~slist$species)) 

### 2.3.2 Variances ####
## Scree plot
plot(pca_onto, main = "PCA")

## Calculate variances
pca.var_onto <- pca_onto$sdev^2 ; pca.var_onto #square of standard devation (variance)
pca.var.per_onto <- round(pca.var_onto/sum(pca.var_onto)*100, 1); pca.var.per_onto 
pca.scores_onto <- c(pca.var.per_onto[1], pca.var.per_onto[2], pca.var.per_onto[3]);pca.scores_onto


### 2.4 GGPLOT #####
#ggplot with Joint configuration data

## Create data frame for ggplot
{
  df$PCAonto <- as.data.frame(pca_onto$x[,1:3]) 
  row.names(df$PCAonto ) <- slist$ind.pure #Naming rows of df$PCA
  df$PCAonto$genus <- slist$genus
  df$PCAonto$species <- slist$species
  df$PCAonto$ind.pure <- slist$ind.pure

}


## 2.4.1. ggplot function ####
library(ggforce)#for hull
library(ggrepel) #geom_text_repel; geom_label_repel

# Write function for ontogenetic morphospace
my.onto.ggplot <- function  (Data, Comp1, Comp2, cex, groups, colors, shapes, Coord){

p <- ggplot2::ggplot(data=Data, aes(x = Data[,Comp1], y =Data[,Comp2], colour=groups, fill = groups, shape=groups))+theme_light()

p <- p + geom_vline(xintercept = 0, colour="darkgrey")+
         geom_hline(yintercept = 0, colour="darkgrey")+
         geom_point(aes(text=paste(slist$ind, slist$species, sep = " - ")), size=cex)+ 
         scale_size_area(max_size = 1)+
         guides(color = guide_legend(override.aes = list(size=5))) #Size of legend symbols       
  
#Shapes & Colors
p <- p + scale_shape_manual(values=shapes)+
         scale_fill_manual(values=colors)+  
         scale_color_manual(values=colors)  

# Theme  
p <- p + theme(legend.title=element_blank(),
               axis.title.x = element_text(color="black", size=12, face="bold"),
               axis.title.y = element_text(color="black", size=12, face="bold"), 
               plot.caption = element_text(hjust = 0))

#Specialized labs    
p <- p + xlab(paste("PC", Comp1, " - ", pca.var.per_onto[Comp1], "%", sep="")) +
         ylab(paste("PC", Comp2, " - ", pca.var.per_onto[Comp2], "%", sep=""))

#Scale & Coordinates
p <- p + scale_x_continuous(breaks = round(seq(round(min(Data[,1]), 1), round(max(Data[,1]), 1), by = 0.05),2))+
         scale_y_continuous(breaks = round(seq(round(min(Data[,1]), 1), round(max(Data[,1]), 1), by = 0.05),2))
p <- p + coord_fixed(ratio = 1, xlim = c(min(Data[,Comp1]),max(Data[,Comp1])), c(min(Data[,Comp2]),max(Data[,Comp2])), expand = TRUE, clip = "on")

#Convex Hull
#Plots$PCAonto<-Plots$PCAonto+ ggforce::geom_mark_hull(aes(fill=factor(slist$species)), expand=0, radius=0, concavity=10) 
return(p)
}

## 2.4.2. PC1/PC2 and PC1/PC3 plot ####
{
windows(width=5, height=4)
Plots$PCAonto_1_2  <- my.onto.ggplot(Data=df$PCAonto, Comp1=1, Comp2=2, cex=2, groups=slist$species, colors=Palette, shapes=Shapes);Plots$PCAonto_1_2 
Plots$PCAonto_1_3  <- my.onto.ggplot(Data=df$PCAonto, Comp1=1, Comp2=3, cex=2, groups=slist$species, colors=Palette, shapes=Shapes);Plots$PCAonto_1_3 
}

## 2.4.3. Combine PC1/PC2 and PC1/PC3 plot ####
library(patchwork)  
{windows(width=10, height=4)

p <- ((Plots$PCAonto_1_2&theme(legend.position = 'none')) | (Plots$PCAonto_1_3))

p <- p+plot_layout(widths = c(1,1))

p <- p+plot_annotation(
  tag_levels = 'A')

P_PCA_onto<-p; P_PCA_onto}

#ggsave(paste("P_PCA_onto", Sys.Date(),".jpeg", sep="_"), dpi=400)
#dev.off()

## Extracting the no. of an individual in the tps.file
#(which(list$ind.pure == "C10137")/11)[11]


#install.packages("geometry")
#install.packages("Rtools")

## 2.4.4. Plotly of onto  ####

{Plots$PCAontoly <- plotly::ggplotly(Plots$PCAonto_1_2)%>%
    layout(title = list(text = paste0("PCA of Ontogenetic Morphospace")))
  Plots$PCAontoly}


 
### 2.4.5. 3D plotly #####
#3D PC plot of combined growth stages

plotly::plot_ly(x=df$PCAonto$PC1, y=df$PCAonto$PC2, z=df$PCAonto$PC3,
                type="scatter3d", mode="markers", #scatter3d pr scatter
                color=df$PCAonto$species,
                colors=Palette)


### 2.4.6. plot3d with hull#####
#hull documentation: https://stackoverflow.com/questions/41145959/picture-convex-hull-in-3d-scatter-plot

#3D plot
{
  rgl::plot3d(df$PCAonto$PC1, df$PCAonto$PC2, df$PCAonto$PC3, col=as.numeric(as.factor(df$PCAonto$species)), box = TRUE,
       xlab="PC1", ylab="PC2", zlab="PC3",
       type ="s", radius = 0.001, size=1)  #or type "p"

# hulls
for (i in c(1,2,3,4,5,6,7,8,10,11,12)){  #species No. 9 (GyWeitsch) has only 3 points (4 are needed)
  ps <- matrix(c(df$PCAonto$PC1[as.numeric(as.factor(slist$species))==i],
                 df$PCAonto$PC2[as.numeric(as.factor(slist$species))==i],
                 df$PCAonto$PC3[as.numeric(as.factor(slist$species))==i]), ncol=3)  # generate points on a sphere

  
  ts.surf <- t(geometry::convhulln(ps))  # see the qhull documentations for the options
  convex <-  rgl::rgl.triangles(ps[ts.surf,1],ps[ts.surf,2],ps[ts.surf,3],
                           col=as.numeric(as.factor(df$PCAonto$species))[as.numeric(as.factor(slist$species))==i],alpha=.6)}
}


